/*
 * Created on 10.10.2008
 */
package Alignment;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import RealValue.Individual;
import Utilities.ClassicEvaluation;
import Utilities.EvalParameter;
import Utilities.Graph;

/**
 * @author Thomas Fober
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         thomas@mathemetik.uni-marburg.de
 */
public class Alignment_old_useUngar {

	public int[][] alignment;
	public double[][] distances;
	public double fitnessPCO;
	public Graph[] graphs;
	
	public Alignment_old_useUngar(){
		
	}
	
	public Alignment_old_useUngar(int[][] star, double fitness, Graph[] graphs, double time, String method, String termination){
		alignment = star;
		fitnessPCO = fitness;
		this.graphs = graphs;
		
	}
	
	public Alignment_old_useUngar(Graph g1, Graph g2, double threshold, Individual overlay){
		Vector<int[]> align = new Vector<int[]>();
		overlay.calcDistances();
		this.distances = overlay.distances;
		Vector<Integer> used0 = new Vector<Integer>();
		Vector<Integer> used1 = new Vector<Integer>();
		
		for(int iter = 0; iter < Math.pow(distances.length, 2); iter++){
			int bestR = 0;
			int bestC = 0;
			double bestvalue = Double.POSITIVE_INFINITY;
			for(int i = 0; i < distances.length; i++){
				if(used0.contains(i)) continue;
				for(int j = 0; j < distances[0].length; j++){
					if(used1.contains(j)) continue;
					if(distances[i][j] < bestvalue){
						bestvalue = distances[i][j];
						bestR = i;
						bestC = j;
					}
				}
			}
			if(bestvalue < threshold){
				//add pair into alignment
				int[] tmp = {bestR,bestC};
				align.add(tmp);
				//remove pair
				used0.add(bestR);
				used1.add(bestC);
			}
		}
		for(int i = 0; i < overlay.molecules[0].size(); i++){
			if(!used0.contains(i)){
				int[] tmp = {i,-1};
				align.add(tmp);
			}
		}
		for(int i = 0; i < overlay.molecules[1].size(); i++){
			if(!used1.contains(i)){
				int[] tmp = {-1, i};
				align.add(tmp);
			}
		}
		alignment = new int[2][align.size()];
		for(int i = 0; i < align.size(); i++){
			alignment[0][i] = align.get(i)[0];
			alignment[1][i] = align.get(i)[1];
		}
		fitnessPCO = ClassicEvaluation.evaluate(alignment, graphs, new EvalParameter());
	}
	
	
	public void save(String path, String name, Graph[] graphs){
		try {
			BufferedWriter alig = new BufferedWriter(new FileWriter(path+File.separator+name+".mga"));
			alig.write("#MGA "+name+"\n");
			alig.write("#Number of graphs  	: "+graphs.length+"\n");
			alig.write("#Calculation method	: PCO\n");//"+this.method+"\n");
			alig.write("#Calculation time  	: n/A\n");//"+this.time+"\n");
			alig.write("#Score             	: "+fitnessPCO+"\n");
			alig.write("#Generations       	: n/A\n");//+this.generations+"\n");
			alig.write("stop reason        	: n/A\n");//+this.reason+"\n");
			alig.write("Scoring parameter		: n/A\n");//+this.parameter+"\n");
			alig.write("#Alignment matrix\n");
			for (int i = 0; i < this.alignment.length; i++) {
				String line = "";
				for (int j = 0; j < this.alignment[0].length-1; j++) {
					line += alignment[i][j] +";";
				}
				line += alignment[i][alignment[0].length-1];
				alig.write(line+"\n");
			}
			for (Graph g : graphs) {
				alig.write("#graph "+g.name+"\n");
				alig.write("#labels\n");
				String line = "";
				for (int i = 0; i < g.nodes.length-1; i++) {
						line += g.nodes[i] +";";
					}
				line += g.nodes[g.nodes.length-1];
				alig.write(line+"\n");
				alig.write("#distance\n");
				for (int i = 0; i < g.distances.length; i++) {
					line = "";
					for (int j = 0; j < g.distances[0].length-1; j++) {
						line += g.distances[i][j] +";";
					}
					line += g.distances[i][g.distances[0].length-1];
					alig.write(line+"\n");
				}
			}
			alig.close();
		}
		catch(IOException e) {
			System.out.println(e);
		}
	}
}
