/*
 * Created on 09.08.2007
 *
 */
package Parser;

import java.io.*;
import java.util.Vector;
import Utilities.Coords;
import Utilities.Graph;

/**
 * @author Thomas Fober & Marco Mernberger
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         {thomas, mernberger}@mathemetik.uni-marburg.de
 */
public class CavityParser implements FileParser{

	private double[][] distances;
	private String[] nodes;
	private String path;
	private Vector<Coords> coords;
	private int numberOfNodes;
	private boolean stop = false;
	
	public CavityParser(){
	}
	
	/**Creates a graph out of a given file
	 * @param path path of file to parse
	 * @param delta cut-off value for edges
	 */
	public Graph createGraph(String path, double delta){
		this.parse(path);
		if(stop) return null;
		this.trimDistance(delta);
		this.createNodes();
		return new Graph(this.numberOfNodes, null, this.distances, this.nodes, this.coords, this.path);
	}
	

	/**Parses a given input file for graph creation.
	 * 
	 */
	private void parse(String path){
			try{
				//open file
				BufferedReader file = new BufferedReader(new FileReader(path));
				Vector<Coords> coords = new Vector<Coords>();
				String next = file.readLine();
				while(next != null){
					next = file.readLine();
					if (next == null) continue;
					next.trim();
					String[] lineC = next.split("\\s+");
					Coords actual = new Coords(Double.parseDouble(lineC[1]), Double.parseDouble(lineC[2]), Double.parseDouble(lineC[3]), lineC[4]);
					coords.addElement(actual);
				}
				int numberNodes = coords.size();
				
				//Creating distance matrix
				double[][] distance = new double[coords.size()][coords.size()];
				for (int i = 0; i < coords.size();i++){
					for (int j = 0; j < coords.size();j++){
						if (i==j) distance[i][j] = 0;
						else distance[i][j] = Math.sqrt(Math.pow((coords.get(i).getX()-coords.get(j).getX()),2) + Math.pow((coords.get(i).getY()-coords.get(j).getY()),2) + Math.pow((coords.get(i).getZ()-coords.get(j).getZ()),2));
					}
				}
				
				//creating bond matrix
				int[][] bonds = null;
				file.close();
				this.distances = distance;
				this.coords = coords;
				this.numberOfNodes = numberNodes;
				this.path = path;
				}
			catch (IOException e){
				System.out.println("File not found error");
			}
			catch(OutOfMemoryError e) {
				System.out.println("graph matrix exceeds memory.");
				this.stop = true;
			}

	}
	
	/**Creates List of nodes labels
	 * 
	 */
	private void createNodes(){
		this.nodes = new String[this.coords.size()];
		for (int i = 0; i < coords.size(); i++){
			this.nodes[i] = coords.get(i).getLabel();
		}
	}
	
	/**Removes edges with distance greater cutoff value
	 * 
	 */
	private void trimDistance(double delta){
		for (int i = 0; i < distances.length; i++){
			for (int j = 0; j < distances[0].length; j++){
				if (distances[i][j]>delta) distances[i][j] = Double.POSITIVE_INFINITY;
			}
		}
	}
		 
}
