/*
 * Created on 08.02.2008
 */
package Parser;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import Utilities.Graph;

/**
 * @author Thomas Fober
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         thomas@mathemetik.uni-marburg.de
 */
public class CommonParser implements FileParser{
	private double[][] distances;
	private String[] nodes;
	private int numberOfNodes;
	
	public CommonParser(){
	}
	
	
	/**Parse a graph for graph-format file
	 * 
	 * @param path path of file storing the graph in graph format
	 * @param delta max distances of two nodes for inserting an edge between them
	 * @return graph object
	 * 
	 */
	public Graph createGraph(String path, double delta){
		this.parse(path);
		return new Graph(this.numberOfNodes, null, this.distances, this.nodes, null, path);
	}
	
	/**Parses a given input file for graph creation.
	 * 
	 */
	private void parse(String path){
		try{
			//open file
			BufferedReader file = new BufferedReader(new FileReader(path));
			//reading node labels
			String f = file.readLine();
			this.nodes = f.trim().split(",");
			this.numberOfNodes = nodes.length;
			this.distances = new double[nodes.length][nodes.length];
			f = file.readLine();
			String[] rows = f.trim().split(";");
			String[] tmp;
			for(int i = 0; i < nodes.length; i++){
				tmp = rows[i].trim().split(",");
				for(int j = 0; j < nodes.length; j++){
					try{
						distances[i][j] = Double.parseDouble(tmp[j]);
					}catch(Exception e){
						distances[i][j] = Double.POSITIVE_INFINITY;
					}
				}
			}
			file.close();
		}catch (IOException e){
			System.out.println("File not found error");
		}
	}
}
