/*
 * Created on 23.10.2007
 */
package RealValue;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import Utilities.Coords;

/**
 * @author Thomas Fober
 *         Philipps-Universitt Marburg
 *         Fachbereich Mathematik und Informatik
 *         thomas@mathemetik.uni-marburg.de
 */
public class Individual implements Cloneable{
	
	//Translation and Rotation
	public double[] object;
	//step-size (endogen ES parameter)
	double[] stepsizes;
	//fitness of individual
	public double fitness;
	//age of individual (for kappa-selection)
	int age;
	//number of stepsizes (n if true, else 1)
	boolean nStepSizes;
	//time (for determine calculation time and also handle stall-time-termination criteria)
	long time;
	//pairwise distances of points (for calc alignment)
	public double[][] distances;
	
	public void calcDistances(){
		Vector<Coords>[] temp;
		temp = new Vector[molecules.length];
		for(int i = 0 ; i < temp.length; i++){
			temp[i] = new Vector<Coords>();
			for(int j = 0; j < molecules[i].size(); j++){
				temp[i].add(molecules[i].get(j).clone());
			}
		}
		for(int i = 1; i < temp.length; i++){
			double[] center = determineCenter(temp[i]);
			rotate(temp[i], center, object[(i-1)*6 + 3], object[(i-1)*6 + 4], object[(i-1)*6 + 5]);
			translate(temp[i], object[(i-1)*6 + 0], object[(i-1)*6 + 1], object[(i-1)*6 + 2]);
		}
		distances = new double[temp[0].size()][temp[1].size()];
		for(int i = 0; i < temp[0].size(); i++){
			for(int j = 0; j < temp[1].size(); j++){
				distances[i][j] = distance(temp[0].get(i), temp[1].get(j));
			}
		}
	}
	
	
	///////////////////////////////////
	//FOR FITNESSFUNCTION CALCULATION//
	///////////////////////////////////
	public Vector<Coords>[] molecules;
	public Vector<Integer[][]> group;
	public String[] typ;
	
	//name of file storing results
	public String name;
	public double relFitnessIter = 0;
	public int numberNodesStartIter = 0;
	
	//for two structures only************************************
	public Vector<double[][]> labeledPairwiseDistances;             //**
	//***********************************************************
	
	public Individual(boolean nSS, Vector<Coords>[] m, String n, Vector<Integer[][]> gr, String[] tE){
		nStepSizes = nSS;
		int nOG = m.length;
		object = new double[6 * (nOG - 1)];
		if(nStepSizes){
			stepsizes = new double[6 * (nOG - 1)];
		}
		if(!nStepSizes){
			stepsizes = new double[1];
		}
		this.molecules = m;
		age = 0;
		name = n;
		if(gr == null && tE == null){
			calculateIndInformation();
		}else{
			//use given information (estimation only needed in the initialization phase)
			group = gr;
			typ = tE;
		}
	}
	
	public int max(int a, int b){
		return a > b ? a : b;
	}
	
	public void calculateIndInformation(){
		//determine all labels appear in the molecules
		Vector<String> allLabels = new Vector<String>();
		group = new Vector<Integer[][]>();
		//run through all molecules and all elements within molecules
		for(int i = 0; i < molecules.length; i++){
			for(int j = 0; j < molecules[i].size(); j++){
				if(!allLabels.contains(molecules[i].get(j).getLabel())){
					allLabels.add(molecules[i].get(j).getLabel());
				}
			}
		}
		//split w.r.t. labels
		for(int i = 0; i < allLabels.size(); i++){
			Vector<Integer>[] tmp = new Vector[molecules.length];
			for(int j = 0; j < molecules.length; j++){
				tmp[j] = new Vector<Integer>();
				for(int k = 0; k < molecules[j].size(); k++){
					if(molecules[j].get(k).getLabel().equals(allLabels.get(i))){
						tmp[j].add(k);
					}
				}
			}
			Integer[][] tmpGroup = new Integer[molecules.length][max(tmp[0].size(), tmp[1].size())];
			for(int j = 0; j < tmpGroup.length; j++){
				for(int k = 0; k < tmpGroup[0].length; k++){
					tmpGroup[j][k] = -1;
				}
			}
			for(int j = 0; j < tmpGroup.length; j++){
				for(int k = 0; k < tmp[j].size(); k++){
					tmpGroup[j][k] = tmp[j].get(k);
				}
			}
			group.add(tmpGroup);
		}
		//generate now typ:
		typ = new String[allLabels.size()];
		for(int i = 0; i < allLabels.size(); i++){
			typ[i] = allLabels.get(i);
		}
	}
	
	public void labeledPairwiseDistances(){
		//calculate overlay
		Vector<Coords>[] temp;
		temp = new Vector[molecules.length];
		for(int i = 0 ; i < temp.length; i++){
			temp[i] = new Vector<Coords>();
			for(int j = 0; j < molecules[i].size(); j++){
				temp[i].add(molecules[i].get(j).clone());
			}
		}
		for(int i = 1; i < temp.length; i++){
			double[] center = determineCenter(temp[i]);
			rotate(temp[i], center, object[(i-1)*6 + 3], object[(i-1)*6 + 4], object[(i-1)*6 + 5]);
			translate(temp[i], object[(i-1)*6 + 0], object[(i-1)*6 + 1], object[(i-1)*6 + 2]);
		}
		if(temp.length > 2){
			System.out.println("Error -- more than two structures");
		}		
		labeledPairwiseDistances = new Vector<double[][]>();
		for(int i = 0; i < group.size(); i++){
			//size of matrix?
			int mol0 = 0;
			int mol1 = 0;
			while(group.get(i)[0][mol0] != -1){
				mol0++;
				if(group.get(i)[0].length == mol0) break;
			}
			while(group.get(i)[1][mol1] != -1){
				mol1++;
				if(group.get(i)[1].length == mol1) break;
			}
			double[][] tmp = new double[mol0][mol1];
			for(int j = 0; j < tmp.length; j++){
				for(int k = 0; k < tmp[j].length; k++){
					tmp[j][k] = distance(temp[0].get(group.get(i)[0][j]), temp[1].get(group.get(i)[1][k]));
				}
			}
			labeledPairwiseDistances.add(tmp);
		}
	}
	
	public Individual clone(){
		Individual result = new Individual(nStepSizes, molecules, "", this.group, this.typ);
		result.object = this.object.clone();
		result.stepsizes = this.stepsizes.clone();
		result.fitness = this.fitness;
		result.age = this.age;
		result.nStepSizes = this.nStepSizes;
		result.molecules = this.molecules;
		result.name = this.name;
		result.group = this.group;
		result.labeledPairwiseDistances = this.labeledPairwiseDistances;
		result.typ = this.typ;
		return result;
	}
	
	private double[] determineCenter(Vector<Coords> coords){
		double[] center = new double[3];
		double numberOfCoords = coords.size();
		double a = 0, b = 0, c = 0;
		for(int i = 0; i < coords.size(); i++){
			a = a + coords.get(i).getX() / numberOfCoords;
			b = b + coords.get(i).getY() / numberOfCoords;
			c = c + coords.get(i).getZ() / numberOfCoords;
		}
		center[0] = a; center[1] = b; center[2] = c;
		return center;
	}
	
	private void translate(Vector<Coords> coords, double x, double y, double z){
		for(int i = 0; i < coords.size(); i++){
			coords.get(i).setX(coords.get(i).getX() + x);
			coords.get(i).setY(coords.get(i).getY() + y);
			coords.get(i).setZ(coords.get(i).getZ() + z);
		}
	}
	
	private double findNearest(boolean row, int position, double[][] array){
		double distance = Double.POSITIVE_INFINITY;
		int i;
		if(row){
			for(i = 0; i < array[position].length; i++){
				distance = distance > array[position][i] ? array[position][i] : distance;
			}
		}else{
			for(i = 0; i < array.length; i++){
				distance = distance > array[i][position] ? array[i][position] : distance;
			}
		}
		return distance;
	}
	
	private void rotate(Vector<Coords> m, double[] center, double angleX, double angleY, double angleZ){
		for(int i = 0; i < m.size(); i++){
			Coords tmp;
			tmp = m.get(i).clone();
			tmp.setX(tmp.getX() - center[0]);
			tmp.setY(tmp.getY() - center[1]);
			tmp.setZ(tmp.getZ() - center[2]);
			m.setElementAt(tmp, i);
			double x, y, z, xx, yy, zz;
			x = m.get(i).getX();
			y = m.get(i).getY() * Math.cos(angleX) + m.get(i).getZ() * Math.sin(angleX);
			z = -m.get(i).getY() * Math.sin(angleX) + m.get(i).getZ() * Math.cos(angleX);
			xx = x * Math.cos(angleY) - z * Math.sin(angleY);
			yy = y;
			zz = x * Math.sin(angleY) + z * Math.cos(angleY);
			x = xx * Math.cos(angleZ) + yy * Math.sin(angleZ);
			y = -xx * Math.sin(angleZ) + yy * Math.cos(angleZ);
			z = zz;
			tmp = new Coords(x, y, z, m.get(i).getLabel());
			tmp.setX(tmp.getX() + center[0]);
			tmp.setY(tmp.getY() + center[1]);
			tmp.setZ(tmp.getZ() + center[2]);
			m.setElementAt(tmp, i);
		}
	}
	
	private Coords nearestMatchNeighbor(Vector<Coords> coords, Coords node, String type){
		double bestValue = Double.POSITIVE_INFINITY;
		Coords best = null;
		for(int i = 0; i < coords.size(); i++){
			//calculate distances
			double dist = Double.POSITIVE_INFINITY;
			if(coords.get(i).getLabel().compareTo(type) == 0){
				dist = distance(node, coords.get(i));
			}
			if(dist < bestValue){
				bestValue = dist;
				best = coords.get(i);
			}
		}
		return best;
	}
	
	private double distance(Coords coordinates1, Coords coordinates2){
	//abs schneller zu berechnen als euklid
		return (Math.abs(coordinates1.getX() - coordinates2.getX()) + Math.abs(coordinates1.getY() - coordinates2.getY()) + Math.abs(coordinates1.getZ() - coordinates2.getZ()));
	}
	
	private double gauss(double x, double sigma){
		return Math.exp(-1.0/2 * Math.pow(x/sigma,2));
	}
	
	private double cauchy(double x, double sigma){
		return 1 / (1 + Math.pow(x/sigma, 2));
	}
	
	private double exponential(double x, double sigma){
		return Math.exp(-x/sigma);
	}
	
	public void evaluate(int membership, double sigma){
		labeledPairwiseDistances();
		fitness = 0;
		for(int i = 0; i < labeledPairwiseDistances.size(); i++){
			if(labeledPairwiseDistances.get(i).length > 0){
				if(membership == 0){				
					for(int j = 0; j < labeledPairwiseDistances.get(i)[0].length; j++){
						fitness += exponential(findNearest(false, j, labeledPairwiseDistances.get(i)), sigma);
					}
					for(int j = 0; j < labeledPairwiseDistances.get(i).length; j++){
						fitness += exponential(findNearest(true, j, labeledPairwiseDistances.get(i)), sigma);
					}
				}
				else if(membership == 1){
					for(int j = 0; j < labeledPairwiseDistances.get(i)[0].length; j++){
						fitness += gauss(findNearest(false, j, labeledPairwiseDistances.get(i)), sigma);
					}
					for(int j = 0; j < labeledPairwiseDistances.get(i).length; j++){
						fitness += gauss(findNearest(true, j, labeledPairwiseDistances.get(i)), sigma);
					}
				}
				else if(membership == 2){
					for(int j = 0; j < labeledPairwiseDistances.get(i)[0].length; j++){
						fitness += cauchy(findNearest(false, j, labeledPairwiseDistances.get(i)), sigma);
					}
					for(int j = 0; j < labeledPairwiseDistances.get(i).length; j++){
						fitness += cauchy(findNearest(true, j, labeledPairwiseDistances.get(i)), sigma);
					}
				}
			}
		}
	}
	
	public void evaluateSmall(){
		labeledPairwiseDistances();
		fitness = 0;
		for(int i = 0; i < labeledPairwiseDistances.size(); i++){
			if(labeledPairwiseDistances.get(i).length > 0){
				if(labeledPairwiseDistances.get(i).length > labeledPairwiseDistances.get(i)[0].length){
					for(int j = 0; j < labeledPairwiseDistances.get(i)[0].length; j++){
						fitness += exponential(findNearest(false, j, labeledPairwiseDistances.get(i)), 1);
					}
				}else{
					for(int j = 0; j < labeledPairwiseDistances.get(i).length; j++){
						fitness += exponential(findNearest(true, j, labeledPairwiseDistances.get(i)), 1);
					}
				}
			}
		}
	}
	
	private double sum(Vector[] m){
		double sum = 0;
		for(int i = 0; i < m.length; i++){
			sum = sum + m[i].size();
		}
		return sum;
	}
	
	private double denominator(Vector[] m){
		double max = m[0].size() > m[1].size() ? m[0].size() : m[1].size(); 
		double min = m[0].size() < m[1].size() ? m[0].size() : m[1].size();
		return 2 * min + (max - min);
	}
	
	public void save(String path, String name){
		try {
			BufferedWriter alig = new BufferedWriter(new FileWriter(path+File.separator+name+".rw"));
			alig.write("#ES "+name+"\n");
			alig.write("#Number of molecules: "+this.molecules.length+"\n");
			alig.write("#Calculation time  	: "+this.time+"\n");
			alig.write("#Score             	: "+this.fitness +"\n");
			alig.write("#Score (normalized)	: "+this.fitness / denominator(molecules) +"\n");
			alig.write("#Angle            	: "+this.object[3]+"  " + this.object[4] + "  " + this.object[5] + "\n");
			alig.write("#Displacement      	: "+this.object[0]+"  " + this.object[1] + "  " + this.object[2] + "\n");
			alig.write("Points in cloud 1   : "+this.molecules[0].size() + "\n");
			alig.write("Points in cloud 2   : "+this.molecules[1].size() + "\n");
			if(this.numberNodesStartIter != 0){
				alig.write("#Start number of nodes: " + this.numberNodesStartIter + "\n");
				alig.write("#RelScore      	: " + this.relFitnessIter + "\n");
			}
			alig.close();
		}
		catch(IOException e) {
			System.out.println(e);
		}
	}
	
	public Individual(String path, Vector<Coords>[] m){
		this.molecules = m;
		try {
			BufferedReader in = new BufferedReader(new FileReader(path));
			this.name = in.readLine().substring(4);
			in.readLine();
			this.time = Long.parseLong(in.readLine().substring(22));
			this.fitness = Double.parseDouble(in.readLine().substring(22));
			in.readLine();
			//angels
			String angles = in.readLine().substring(22);
			object = new double[6];
			object[3] = Double.parseDouble(angles.split("  ")[0]);
			object[4] = Double.parseDouble(angles.split("  ")[1]);
			object[5] = Double.parseDouble(angles.split("  ")[2]);
			String displacement = in.readLine().substring(22);
			object[0] = Double.parseDouble(displacement.split("  ")[0]);
			object[1] = Double.parseDouble(displacement.split("  ")[1]);
			object[2] = Double.parseDouble(displacement.split("  ")[2]);
		}
		catch (IOException e) {
			System.out.println(e);
		}
	}
	
	public void save(String path) {
		this.save(path, this.name);
	}
	
	public void transform(){
		rotate(molecules[1], determineCenter(molecules[1]), this.object[3], this.object[4], this.object[5]);
		translate(molecules[1], this.object[0], this.object[1], this.object[2]);
		this.object = new double[6];
	}
}
